const express = require('express');
const app = express();
const urlencodedParser = express.urlencoded({
	extended: true
});
const session = require('express-session');
const nunjucks = require('nunjucks');
const bcrypt = require('bcryptjs');

nunjucks.configure('templates', {
	autoescape: true,
	express: app
});

const cookieName = "clubsid"; // Session ID cookie name, use this to delete cookies too.
app.use(session({
	secret: 'website development CSUEB',
	resave: false,
	saveUninitialized: false,
	name: cookieName // Sets the name of the cookie used by the session middleware
}));

// My fake user and event data
const users = require('./secStudents.json');
const events = require('./events.json');
// Change from string dates to Date()
for (let event of events) {
	event.date = new Date(event.date);
}


// This initializes session state
const setUpSessionMiddleware = function (req, res, next) {
	console.log(`session object: ${JSON.stringify(req.session)}`);
	console.log(`session id: ${req.session.id}`);
	if (!req.session.user) {
		req.session.user = {
			loggedin: false
		};
	}
	next();
};

app.use(setUpSessionMiddleware);

// Use this middleware to restrict paths to only logged in users
const checkLoggedInMiddleware = function (req, res, next) {
	if (!req.session.user.loggedin) {
		res.render("Forbidden.njk");
	} else {
		next();
	}
};

// Available to all visitors
app.get('/', function (req, res) {
	res.render('index.njk', {
		user: req.session.user
	});
});

// Available to all visitors
app.get('/events', function (req, res) {
	res.render('events.njk', {
		events: events,
		user: req.session.user
	});
});

// Available to all visitors
app.get('/loginForm', function (req, res) {
	res.render('logon.njk', {
		user: req.session.user
	});
});

// Only available to logged in members
app.get('/users', checkLoggedInMiddleware, function (req, res) {
	res.render('users.njk', {
		users: users,
		user: req.session.user
	});
});

// Only available to logged in members
app.get('/addEventForm', checkLoggedInMiddleware, function (req, res) {
	res.render('addEvent.njk', {
		user: req.session.user
	});
});

// Only available to logged in members
app.get('/addEvent', checkLoggedInMiddleware, function (req, res) {
	let temp = req.query;
	console.log(temp);
	// Note need to check input here to prevent injection attacks
	let event = {
		name: temp.event_name,
		day: temp.event_day,
		date: new Date(temp.event_date)
	};
	events.push(event);
	res.redirect('/events');
});

// Available to all visitors
app.post('/logon', express.urlencoded({extended:true}), function (req, res) {
	console.log(req.body);
	let email = req.body.email;
	let password = req.body.password;
	// Find user
	let auser = users.find(function (user) {
		return user.email === email
	});
	if (!auser) {// Not found
		res.render("loginError.njk");
		return;
	}
	let verified = bcrypt.compareSync(password, auser.passHash);
	if (verified) {
		// Upgrade in priveledge, should generate new session id
		// Save old session information if any, create a new session
		let oldInfo = req.session.user;
		req.session.regenerate(function (err) {
			if (err) {
				console.log(err);
			}
			req.session.user = Object.assign(oldInfo, auser, {
				loggedin: true
			});
			res.render("welcome.njk", {user: auser});
		});
	} else {
		res.render("loginError.njk");
	}
});

// Available to all visitors
app.get('/logout', function (req, res) {
	let options = req.session.cookie;
	req.session.destroy(function (err) {
		if (err) {
			console.log(err);
		}
		res.clearCookie(cookieName, options); // the cookie name and options
		res.render("goodbye.njk");
	})
});

const host = '127.0.0.1';
const port = '3434';
app.listen(port, host, function () {
	console.log("Club session server listening on IPv4: " + host +
		":" + port);
});
